/********************************************************************************
*
*--------------------------------------------------------------------------------
*--        Copyright Magnus Nilsson and SPL, Griffith University 2001          --
*--------------------------------------------------------------------------------
*--
*--
*--
*-- The copyright of the source program herein is the property of Magnus Nilsson 
*-- and Signal Processing Laboratory, Griffith University, Australia. 
*-- The program may be used and/or copied only with the written permission from 
*-- Magnus Nilsson and Signal Processing Laboratory, Griffith University or in 
*-- accordance with theterms and conditions stipulated in the agreement/contract 
*-- under which the program has been supplied.
*--
*--
*--------------------------------------------------------------------------------
*--------------------------------------------------------------------------------
*--
*-- Project    : VQ-speaker verification
*-- Title      : Speaker Verification
*-- File name  : SpeakerVerification.java
*-- Revision   : pa1
*-- Date       : 25-09-2001
*-- Prepared   : Magnus Nilsson
*-- Responsible:
*-- Description: 
*-- -----------------------------------------------------------------------------
*-- Revision history: (all revisions included)                                 --
*-- -----------------------------------------------------------------------------
*-- Rev:    Initials:    Date:       Modification                              --
*-- -----------------------------------------------------------------------------
*-- pa1     MN           25-09-01    Final revision                            --
*-- -----------------------------------------------------------------------------
********************************************************************************/
import java.awt.*;
import java.awt.Color.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.colorchooser.*;
import javax.swing.filechooser.*;
import javax.accessibility.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.plaf.metal.*;
import javax.sound.sampled.*;
import java.applet.*;
import java.net.*;
import java.lang.Object;
import java.io.*;

public class SpeakerVerification extends JFrame {
	public static int WIDTH = 500;
	public static int HEIGHT =500;
	public int counter = 0;
	public static String TITLE = "Speaker Verification @ SPL";	
	boolean test = false;

	RecordSpeech recordSpeech = new RecordSpeech();
	dialogframe frameWindow;

	Container frameContainer;
	JPanel[] panels = new JPanel[2];
	JMenuBar menuBar = new JMenuBar();
	JMenu fileMenu = new JMenu("File");
	JMenuItem fileExit = new JMenuItem("Exit");
	JMenu aboutMenu = new JMenu("About");
	JMenuItem aboutHelp = new JMenuItem("Help");
	JMenuItem aboutMasters = new JMenuItem("Masters Project");
	JMenuItem aboutCredit = new JMenuItem("Credit");

	Icon headerIcon = new ImageIcon("images/magnus2_01.gif");

	ImageIcon buttonAddIcon1 = new ImageIcon("images/magnus2_02-normal.gif"); //normal
	ImageIcon buttonAddIcon2 = new ImageIcon("images/magnus2_02-down.gif"); //pressed
	ImageIcon buttonAddIcon3 = new ImageIcon("images/magnus2_02-over.gif"); //roll_over
	ImageIcon buttonAddIcon4 = new ImageIcon("images/magnus2_02-normal.gif"); //disabled

	ImageIcon buttonVerifyIcon1 = new ImageIcon("images/magnus2_03-normal.gif");
	ImageIcon buttonVerifyIcon2 = new ImageIcon("images/magnus2_03-down.gif");
	ImageIcon buttonVerifyIcon3 = new ImageIcon("images/magnus2_03-over.gif");
	ImageIcon buttonVerifyIcon4 = new ImageIcon("images/magnus2_03-disabled.gif");

	JButton addButton = new JButton(buttonAddIcon1);
	JButton verifyButton = new JButton(buttonVerifyIcon1);

	JTextPane jtextpane = new JTextPane();

	MetalLookAndFeel metalLF = new MetalLookAndFeel(); 

	public SpeakerVerification() {
		super(TITLE);
		buildGUI();
		setupEventHandlers();
		recordSpeech.paraport(0); //Set the parallel port extension output to off
		recordSpeech.makeTriang(); //Make triangelfiler for the MFCC's
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int w = 500;
        int h = 500;
        setLocation(screenSize.width/2 - w/2, screenSize.height/2 - h/2);
		setSize(w, h);
		setBackground(Color.white);
		show();		
	}
 
	void buildGUI() {
		setupMenuBar();
		layoutComponents();
		readUsers();		
	}

	void setupMenuBar() {
		fileMenu.add(fileExit); 
		aboutMenu.add(aboutHelp); 
		aboutMenu.add(aboutMasters); 
		aboutMenu.add(aboutCredit); 
		menuBar.add(fileMenu);
		menuBar.add(aboutMenu);
		setJMenuBar(menuBar);
	}

	public void layoutComponents() {

		for(int i=0;i<panels.length;++i)
		panels[i] = new JPanel();
		frameContainer = getContentPane();
		frameContainer.setLayout(new GridLayout(2,1));
		panels[0].setLayout(new GridLayout(1,1));
		panels[0].add(jtextpane);		
		jtextpane.insertIcon(headerIcon);
		jtextpane.setEditable(false);
		jtextpane.setMargin(new Insets(0,0,0,0));
		panels[1].setLayout(new GridLayout(1,2));
		addButton.setBackground(Color.white);
		addButton.setForeground(Color.white);
		addButton.setPressedIcon(buttonAddIcon2);
		addButton.setRolloverIcon(buttonAddIcon3);
		addButton.setDisabledIcon(buttonAddIcon4);
		addButton.setFocusPainted(false);
		addButton.setBorderPainted(false);
		addButton.setContentAreaFilled(false);
		addButton.setMargin(new Insets(0,0,0,0));
		verifyButton.setBackground(Color.white);
		verifyButton.setForeground(Color.white);
		verifyButton.setPressedIcon(buttonVerifyIcon2);
		verifyButton.setRolloverIcon(buttonVerifyIcon3);
		verifyButton.setDisabledIcon(buttonVerifyIcon4);
		verifyButton.setFocusPainted(false);
		verifyButton.setBorderPainted(false);
		verifyButton.setContentAreaFilled(false);
		verifyButton.setMargin(new Insets(0,0,0,0));
		panels[1].add(addButton);
		panels[1].add(verifyButton);

		for(int i=0;i<panels.length;++i) frameContainer.add(panels[i]);
	}

	void setupEventHandlers() {
		addWindowListener(new WindowHandler());
		//Menu
		fileExit.addActionListener(new MenuItemHandler());
		aboutHelp.addActionListener(new MenuItemHandler());
		aboutMasters.addActionListener(new MenuItemHandler());
		aboutCredit.addActionListener(new MenuItemHandler());
		//Buttons
		addButton.addActionListener(new ButtonHandler());
		verifyButton.addActionListener(new ButtonHandler());
		//Change Look and Feel
		try {
			UIManager.setLookAndFeel(metalLF);	//Metal look and feel
			SwingUtilities.updateComponentTreeUI(SpeakerVerification.this);
		}catch(Exception ex){
			System.out.println(ex);
		}
	}

	void readUsers() {
			try	{
				recordSpeech.users = 0;
				BufferedReader myIn = new BufferedReader(new InputStreamReader(System.in));
				String namn = "user/userinfo.info";
				BufferedReader inFil = new BufferedReader(new FileReader(namn));
				while (true){
					String line = inFil.readLine();
					if (line == null) {						
						break;
					}
					recordSpeech.userName[recordSpeech.users] = line;
					recordSpeech.users ++;
				}
				if (recordSpeech.users == 0)	{
					verifyButton.setEnabled(false);
				}
				inFil.close();
			} catch (IOException ex) {
				ex.printStackTrace(); 
				return;		
			}				
		}

	public static void main(String[] args) {
		SpeakerVerification app = new SpeakerVerification();	
	}

	public class WindowHandler extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	}

	public class MenuItemHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand();
			if(cmd.equals("Exit")) System.exit(0);
			else if (cmd.equals("Help")) {
				//Show picture with link to help
				recordSpeech.showInteraction("Help", 4, false);								
			}
			else if (cmd.equals("Masters Project")) {
				//Some info about the Master program and the features (MACV)
				recordSpeech.showInteraction("Master Project", 5, false);				
			}
			else if (cmd.equals("Credit")) {
				//Credit to people
				recordSpeech.showInteraction("Credit", 6, false);				
			}
		}
	}

	public class ButtonHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == addButton) addUser();
			else if (e.getSource() == verifyButton) verifyUser();
		}
	}

	public void addUser() {
		recordSpeech.recordingSilence();
		//dialogframe frameWindow;
		frameWindow = new dialogframe("Add User", true); //true = add user
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int w = 272;
        int h = 300;
        frameWindow.setLocation(screenSize.width/2 - w/2, screenSize.height/2 - h/2);
        frameWindow.setSize(w, h);
		frameWindow.setVisible(true);
	}
	
	public void verifyUser() {
		recordSpeech.recordingSilence();
		//dialogframe frameWindow;
		frameWindow = new dialogframe("Verify User", false); //false = verify user
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int w = 272;
        int h = 300;
        frameWindow.setLocation(screenSize.width/2 - w/2, screenSize.height/2 - h/2);
        frameWindow.setSize(w, h);
		frameWindow.setVisible(true);
	}

	public class dialogframe extends Frame {

		boolean adduser;
		boolean told = false;
		boolean jumpout = false;

		JPanel[] dialogPanels = new JPanel[3];
		TextField text1 = new TextField(20);

		ImageIcon yesButton1 = new ImageIcon("images/user_01.gif"); //normal
		ImageIcon yesButton2 = new ImageIcon("images/user_01-down.gif"); //pressed
		ImageIcon yesButton3 = new ImageIcon("images/user_01-over.gif"); //mouse over
		ImageIcon yesButton4 = new ImageIcon("images/user_01.gif"); //disabled
		ImageIcon noButton1 = new ImageIcon("images/user_02.gif"); //normal
		ImageIcon noButton2 = new ImageIcon("images/user_02-down.gif"); //presed
		ImageIcon noButton3 = new ImageIcon("images/user_02-over.gif"); //mouse over
		ImageIcon noButton4 = new ImageIcon("images/user_02.gif"); //diabled
		Icon speakIcon = new ImageIcon("images/speak_04.gif");
		Icon extraIcon = new ImageIcon("images/extra_01.gif");
		
		JButton yesButton = new JButton(yesButton1);
		JButton noButton = new JButton(noButton1);
		JTextPane jtextpane = new JTextPane();
		JTextPane jtextpane2 = new JTextPane();
		JTextPane jtextpane3 = new JTextPane();

		dialogframe(String title, boolean add_verify){ 			
			super(title);
			//boolean add_verify, case true => add
			adduser = add_verify;
			for(int i=0;i<dialogPanels.length;++i)
			dialogPanels[i] = new JPanel();
			setLayout(new GridLayout(3,1));

			dialogPanels[0].setLayout(new GridLayout(1,1));
			dialogPanels[0].add(jtextpane);
			jtextpane.insertIcon(speakIcon);
			jtextpane.setEditable(false);
			jtextpane.setMargin(new Insets(0,0,0,0));
			
			dialogPanels[1].setLayout(new GridLayout(3,1));
			dialogPanels[1].add(jtextpane2);
			jtextpane2.insertIcon(extraIcon);
			jtextpane2.setEditable(false);
			jtextpane2.setMargin(new Insets(0,0,0,0));
			dialogPanels[1].add(text1);
            text1.setText("Enter username");
			text1.selectAll();
			dialogPanels[1].add(jtextpane3);
			jtextpane3.insertIcon(extraIcon);
			jtextpane3.setEditable(false);
			jtextpane3.setMargin(new Insets(0,0,0,0));			
			          	
			dialogPanels[2].setLayout(new GridLayout(1,2));
			yesButton.setPressedIcon(yesButton2);
			yesButton.setRolloverIcon(yesButton3);
			yesButton.setDisabledIcon(yesButton4);
			yesButton.setFocusPainted(false);
			yesButton.setBorderPainted(false);
			yesButton.setContentAreaFilled(false);
			yesButton.setMargin(new Insets(0,0,0,0));
			noButton.setPressedIcon(noButton2);
			noButton.setRolloverIcon(noButton3);
			noButton.setDisabledIcon(noButton4);
			noButton.setFocusPainted(false);
			noButton.setBorderPainted(false);
			noButton.setContentAreaFilled(false);
			noButton.setMargin(new Insets(0,0,0,0));
			dialogPanels[2].add(yesButton);
			dialogPanels[2].add(noButton);

			for(int i=0;i<dialogPanels.length;++i) add(dialogPanels[i]);
		
			yesButton.addActionListener(new DialogHandler());
			noButton.addActionListener(new DialogHandler());			
			addWindowListener(new DialogWindowHandler());		
		}

		public class DialogHandler implements ActionListener {
			public void actionPerformed(ActionEvent e) {
				String enterUser = "Enter username";	
				if (e.getSource() == yesButton) {	
					String username = text1.getText();
					//told = false;					
					if (username.equals(enterUser)) {
						System.out.println("Please enter correct name!!!");
					}
					else if (adduser == true) {						
						for (int i = 0; i < recordSpeech.users; i++) {						
							if (username.equals(recordSpeech.userName[i])) {								
								if (told == false) {
									told = true;
									jumpout = false;
									//Show picture saying they will overwrite old user if they continue	
									recordSpeech.showInteraction("Bad username?!", 8, false);
									System.out.println("Overwrite old user??");
									break;
								}
								else { 
									jumpout = true;
									break;
								}																							
							}
							else {
								jumpout = true;
							}
						}
						if ((jumpout == true) || (recordSpeech.users == 0)) {
							jumpout = false;
							told = false;
							username = username;
							boolean test = false;
							recordSpeech.ok = false;
							test = recordSpeech.recording(username, adduser);												
							if (test == true) {
								setVisible(false);
								recordSpeech.silence = false;
								if (recordSpeech.users > 0) {
									verifyButton.setEnabled(true);
								}
							}								
						}
					}										
					else {
						boolean okuser = false;
						for (int i = 0; i < recordSpeech.users; i++) {						
							if (username.equals(recordSpeech.userName[i])) {				
								okuser = true;
							}
						}
						if (okuser == true) {						
							told = false;
							username = username;
							boolean test = false;
							recordSpeech.ok = false;
							test = recordSpeech.recording(username, adduser);												
							if (test == true) {
								setVisible(false);
								recordSpeech.silence = false;
								if (recordSpeech.users > 0) {
									verifyButton.setEnabled(true);
								}
							}												
						}
						else { 
							//Case user verify user and user not in database
							recordSpeech.showInteraction("Bad username?", 9, false);
						}
					}				
				}
				else if (e.getSource() == noButton) {
					setVisible(false);
					recordSpeech.silence = false;
				}
			}
		}

		public class DialogWindowHandler extends WindowAdapter {
			public void windowClosing(WindowEvent e) {
				setVisible(false);
				recordSpeech.silence = false;
			}
		}
	}
}